'''Partendo da una parola e' possibile, a volte, formare altre parole rimuovendo
una lettera. Ad esempio, partendo dalla parola "parlare" possiamo rimuovere la
"l" ottenendo "parare", rimuovendo poi la "p" otteniamo "arare" e togliendo
l'iniziale "a" si ha "rare". Partendo da una parola possiamo considerare tutte
le parole che si ottengono rimuovendo una lettera e poi possiamo ripetere la
stessa operazione a partire da quest'ultime e cosi' via ottenendo un albero di
parole. 
Ad esempio, nel  file palla.pdf e' riportato  l'albero che si ottiene partendo 
dalla parola "palla" facendo riferimento alle parole lecite contenute 
nel file parole.txt.
Come si vede anche dall'esempio di "palla", nessun nodo dell'albero deve 
avere due figli con la stessa parola mentre una stessa parola puo' apparire 
piu' volte nell'albero.


1) Definire una classe Nodo che rappresenta un nodo di un albero di parole.
 
2) scrivere una funzione es(s, fname)  ricorsiva o che faccia 
   uso di funzioni ricorsive che 
 - prende in input una parola s e il percorso al file delle parole lecite. 
 - facendo riferimento al file di parole lecite   GENERA l'albero di parole 
   con radice la parola s e RITORNA infine il nodo radice dell'albero 
   (di tipo Node). 

3) scrivere una funzione differenti(nodo) ricorsiva o che faccia uso di 
   funzioni ricorsive, che 
   – prende in input la radice di un albero di parole 
   - restituisce l'insieme delle parole dell'albero.
Nel caso dell'albero generato da 'palla'  l'insieme restituito deve essere
{'la', 'all', 'alla', 'palla', 'pala', 'al', 'ala', 'a'}


4) scrivere una funzione lunghe(nodo,k) ricorsiva o che faccia uso 
   di funzioni ricorsive che 
   – prende in input la radice di un albero di parole e un intero k 
   – restituisce l'insieme delle parole dell'albero che hanno lunghezza k.
Nel caso dell'albero generato da 'palla' se k=3 l'insieme deve 
essere {'ala','all'}

5) scrivere una funzione frequenti(nodo) ricorsiva o che faccia uso di 
funzioni ricorsive che 
   – prende in input la radice di un albero di parole 
   – restituisce l'insieme di parole che compaiono piu' spesso nell'albero.
Nel caso dell'albero generato da 'palla' l'insieme conterra' come unica 
parola la lettera a.

'''

class Nodo:
    def __init__(self, x):
        self.nome = x
        self.figli = [] #lista dei nodi figli



def es(s, fname):
    '''la funzione costruisce l'albero a partire da s utilizzando parole del file 
    di testo fname e ne restituisce la radice'''
    with open(fname) as f: 
        listap=f.read().split()
    return es1(s, listap)


def es1(s, listap):
    '''la funzione costruisce l'albero a partire da s utilizzando parole della lista 
    listap  e ne restituisce la radice'''
    nodo = Nodo(s)
    lista_figli=trova_figli(s,listap)
    for x in lista_figli:
        nodo1=es1(x,listap)
        nodo.figli.append(nodo1)
    return nodo
 
 
def trova_figli(s,listap):
    ''' restituisce la lista delle parole in listap che possono essere figli di s'''
    lista=[]
    for i in range(len(s)):
        s1 = s[:i] + s[i+1:]
        if s1 in listap and s1 not in lista:
            lista.append(s1) 
    return lista
    
# >>> r1=es('palla','parole.txt')
# >>> r2=es('marea','parole.txt')

#------------------------------------------------------------------------------------


def differenti(nodo):
    '''Ritorna l'insieme delle parole presenti nell'albero di parole di radice nodo'''
    ins={nodo.nome}
    for x in nodo.figli:
       ins=ins|differenti(x)
    return ins
    
# >>> differenti(r1)
# {'la', 'all', 'alla', 'palla', 'pala', 'al', 'ala', 'a'}
# >>> differenti(r1)
# {'mare', 'rea', 'are', 'e', 'ara', 'area', 'mar', 'a', 'marea', 'ma', 're'}


#----------------------------------------------------------------------------------

def lunghe(nodo,k):
    ''' ritorna l'insieme con le parole lunghe k nell'albero di parole di radice nodo'''
    if len(nodo.nome)< k: return set()
    if len(nodo.nome)==k: return {nodo.nome}
    ins=set()
    for x in nodo.figli:
        ins=ins| lunghe(x,k)
    return ins

# >>> lunghe(r,3)
# {'ala', 'all'}
# >>> lunghe(r2,3)
# {'ara', 'mar', 'rea', 'are'}


#----------------------------------------------------------------------------------

def frequenti(nodo):
    '''Ritorna le parole piu' frequenti nell'albero di parole di radice nodo'''
    d = frequenti1(nodo)
    m=max(d.values())
    ins={x for x in d if d[x]==m}
    return ins

def frequenti1(nodo):
    '''Ritorna un dizionario con il numero di occorrenze per ogni parola presente 
    nell'albero di parole di radice nodo'''
    d={}
    d[nodo.nome] =1
    for figlio in nodo.figli:       
        d1 = frequenti1(figlio)
        for x in d1:
            if x in d: d[x] += d1[x]
            else: d[x]=d1[x]
    return d

# >>> frequenti(r)
# {'a'}
#>>> frequenti(r1)
#{'e', 're'}


#---------------------------------------------------------------------------------




